<?php
/*
phpdtaus - builder and parser for dtaus format

Copyright (C) 2009 Lukas Mueller

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/

class Dtaus_Reader
{
	/**
	 * @var	string String with DTAUS-Content
	 */
	protected $content = '';
	
	/**
	 * @param	$content	string	string with the DTAUS-Content
	 * @return void
	 */
	public function setContent($content)
	{
		$this->content = $content;
	}
	
	/**
	 *	Get a part out of the content string
	 * 	@param	$start	integer			start
	 *	@param	$offset	integer			offset
	 *	@param	$isInt	bool			Cast to integer (removes leading zeros)
	 *	@return			string|integer	integer or string with requested data
	 */
	protected function _getPart($start, $offset = 0, $isInt = 0)
	{
		$data = substr($this->content, $start, $offset); // copy out of text
		$data = trim($data); // remove spaces
		
		if ($isInt) // if value should be an integer
			$data = (int)$data; // cast to integer, so we loose the leading zeros
		return $data;
	}
	
	/**
	 * Returns the found data
	 * @return	array		Array with found data
	 */
	public function getData()
	{
		$extractedData = array();
		$textlength = strlen($this->content);
		
		if ($textlength % 128 != 0) // data length must be a multiple of "128"
		{
			throw new Exception('content length is not a multiple of 128');
		}
		else
		{		
			$extractedData['type'] 			= $this->_getPart(5, 1);
			$extractedData['senderName'] 	= $this->_getPart(23, 27);
			$extractedData['senderNumber'] 	= $this->_getPart(60, 10, 1);
			$extractedData['senderBank'] 	= $this->_getPart(7, 8, 1);
			$extractedData['creationDate'] 	= $this->_getPart(50, 6);
			
			$extractedData['checksums']					= array();
			$extractedData['checksums']['entries']		= $this->_getPart(-118, 7, 1);
			$extractedData['checksums']['accountsum']	= $this->_getPart(-98, 17, 1);
			$extractedData['checksums']['banksum']		= $this->_getPart(-81, 17, 1);
			$extractedData['checksums']['amountsum']	= $this->_getPart(-64, 11, 1) . '.' . $this->_getPart(-53, 2, 1);
			$extractedData['entries'] = array();
			
			// read out the entries
				// checksum array for entries
				$checksums 					= array();
				$checksums['entries']		= 0;
				$checksums['accountsum']	= 0;
				$checksums['banksum']		= 0;
				$checksums['amountsum']		= 0;
				$x = 128; // will be extended with the position
				for ($i = 0; $i < $extractedData['checksums']['entries']; $i++)
				{
					$extensions = ($this->_getPart($x, 4, 1) - 187) / 29;
					if (!is_int($extensions))
					{
						throw new Exception ('extension count of entry number ' . ($i + 1) . ' is not valid (is ' . $extensions . ')');
						return;
					}
					$entry 					= array();
					$entry['bank']			= $this->_getPart($x + 13, 8, 1);
					$entry['account']		= $this->_getPart($x + 21, 10, 1);
					$entry['customerid']	= $this->_getPart($x + 31, 13, 1);
					$entry['amount']		= $this->_getPart($x + 79, 9, 1) . '.' . $this->_getPart($x + 88, 2, 1);
					$entry['name']			= $this->_getPart($x + 93, 27);
					$entry['references']	= array($this->_getPart($x + 155, 27));
					
					$extractedData['entries'][] = $entry;
					// checksums
					$checksums['entries']++;
					$checksums['accountsum']	+= $entry['account'];
					$checksums['banksum']		+= $entry['bank'];
					$checksums['amountsum'] 	+= $entry['amount'];
				}
		}
		
		// check checksums
		foreach($extractedData['checksums'] as $checksumKey => $checksumValue)
		{
			if ($checksumValue != $checksums[$checksumKey])
				throw new Exception('checksum ' . $checksumKey . ' is invalid (is "' . $checksums[$checksumKey] . '"; should be "' . $checksumValue . '"');
		}
		
		return $extractedData;
	}
	
}
?>